;**********************************************************************
;  
;                                                                     *
;    Filename:	    us_pic.asm                                           *
;    Date:          2011-12-11                                                   *
;    File Version:  v2.0                                                  *
;                                                                     *
;    Author:     	Aloys                                                     *
;    Company:                                                         *
;                                                                     * 
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P16F73.INC                                       *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Notes:                                                           *
;                                                                     *
;**********************************************************************


	list		p=16f73		; list directive to define processor
	#include	<p16f73.inc>	; processor specific variable definitions
	

	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _HS_OSC
;	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _XT_OSC

; '__CONFIG' directive is used to embed configuration data within .asm file.
; The lables following the directive are located in the respective .inc file.
; See respective data sheet for additional information on configuration word.






;***** VARIABLE DEFINITIONS
w_temp		EQU	0x20		; variable used for context saving
w_temp1		EQU	0xA0		; reserve bank1 equivalent of w_temp 
status_temp	EQU	0x21		; variable used for context saving
pclath_temp	EQU	0x22		; variable used for context saving
buf			equ	0x23		; puffer fr spi
count		equ	0x24		; zaehler fr spi
count1		equ 0x25		; zaehler fuer timer
us10_count  equ 0x26		; zaehler fuer 10 us_wait
ms_count    equ 0x27		; zaehler fuer ms_wait
s_count		equ 0x28 		;zaehler fuer s_wait
level       equ 0x29	    ; Laufzeitzahler fuer Level
ac_levell   equ 0x2a		; integrierter Level niederwertig
ac_levelh   equ 0x2b		; integrierter Level hochwertig
true_level  equ 0x2c        ; Tatschlicher Fllstand
digit1      equ 0x2d        ; digit 1 des Fllstandes
digit2		equ 0x2e		; digit 2 des Fllstandes
digit3      equ 0x2f        ; digit 3 des Fllstandes
temp1       equ 0x30        ; lokaler Puffer
temp2       equ 0x31		; lokaler Puffer
temp3       equ 0x32		; lokaler Puffer
err_flag    equ 0x33        ; Flag fr Fehler bei der Messung
err_mod     equ 0x34        ; Fehlermode zu Kurz oder zu Lang

; konstanten festlegen

#define	SCL		PORTC,3		;Takt
#define	SDA		PORTC,4		;Daten
#define SC		PORTC,7     ;LCD Chip Select
#define ID      PORTC,6		;LCD Intrukctions/Daten
#define LED     PORTC,5     ; LED fr signalisierung
#define US_S    PORTB,3     ; Ausgang zu Steuern des Ultraschall Gebers
#define US_E    PORTB,4		; Eingang zum erfassen des Echos
#define CMC     D'115'		; Initialisierung TRM0 Zhler 256 -(147-6)
#define UE		D'190'		; Untere Enstellung


;**********************************************************************
	ORG     0x000             ; processor reset vector
	clrf    PCLATH            ; ensure page bits are cleared
  	goto    main              ; go to beginning of program


	ORG     0x004             ; interrupt vector location

	movwf   w_temp            ; save off current W register contents
	movf	STATUS,w          ; move status register into W register
	bcf     STATUS,RP0        ; ensure file register bank set to 0
	movwf	status_temp       ; save off contents of STATUS register
	movf	PCLATH,w	  	  ; move pclath register into w register
	movwf	pclath_temp	  	  ; save off contents of PCLATH register
;
;-- eigentliche Interruptbehandlung
;  
    bcf     INTCON,2		  ; Rcksetzen Interruptflag
    movlw   CMC				  ; Wert fr Timer
    movwf   TMR0		      ; Timer Zaehler setzen 
    incf    level,1			  ; incrementiert Level

	movf	pclath_temp,w	  ; retrieve copy of PCLATH register
	movwf	PCLATH		  	  ; restore pre-isr PCLATH register contents
	movf    status_temp,w     ; retrieve copy of STATUS register
	movwf	STATUS            ; restore pre-isr STATUS register contents
	swapf   w_temp,f
	swapf   w_temp,w          ; restore pre-isr W register contents
	retfie                    ; return from interrupt

;****************************************************************************
; Display Level: Anzeige des Wertes in der Variablen true_level
; 
;****************************************************************************

display:
  clrf  digit1          ; digit 1 initialisieren
digit1_loop:
  movlw  D'100'			; erst hunderter stellen
  subwf  true_level,0   ;  
  btfss  STATUS,0		; prfen of Subtraktion zu berlauf gefhrt hat
  goto   display_1		
  movwf  true_level		; kein berlauf
  incf   digit1,1       
  goto   digit1_loop    ; nchste Schleife
display_1				; jetzt digit2
 clrf  digit2           ; digit 1 initialisieren
digit2_loop:
  movlw  D'10'			; erst zehner stellen
  subwf  true_level,0   ;  
  btfss  STATUS,0		; prfen of Subtraktion zu berlauf gefhrt hat
  goto   display_2		
  movwf  true_level		; kein berlauf
  incf   digit2,1       
  goto   digit2_loop    ; nchste Schleife
display_2
  movfw  true_level		; der Rest sind die Einer
  movwf  digit3  
;
; Ausgabe 
;
  bcf		SCL			; clock low
  bcf		SC			;chip select   
  bcf		ID			; Instructions
  movlw	B'00000010'	; Curser RESET
  call    spi_tx
  bsf		ID			; Daten
  movlw   B'00110000' ;
  iorwf   digit1,0	;
  call    spi_tx
  movlw   B'00110000' ;
  iorwf   digit2,0	;
  call    spi_tx
  movlw   B'00110000' ;
  iorwf   digit3,0	;
  call    spi_tx
  bsf		SC
  return

;****************************************************************************
; Display_err: Anzeige dass Messung fehlerhaft war
; 
;****************************************************************************
display_error:
  bcf		SCL			; clock low
  bcf		SC			;chip select   
  bcf		ID			; Instructions
  movlw	B'00000010'		; Curser RESET
  call    spi_tx
  bsf		ID			; Daten
  movlw   B'01000101' 	;
  call    spi_tx
  movfw   err_mod     	; zweite Char ist Fehlermode E=early L=late
  call    spi_tx
  movlw	B'00000000'	    ; Sende 'Blaenk'
 call    spi_tx
  bsf		SC
  return	

main
    call ini_ports
    call ini_lcd
    
main_loop:
    bcf   LED
    movlw D'1'
    movwf s_count
    call  s_wait
    call messen
    movlw D'1'
    movwf s_count
    call  s_wait
    goto main_loop

; remaining code goes here


;****************************************************************************
; ini_ports initialisiert die ports
;
;****************************************************************************

ini_ports:
    bsf   STATUS,RP0   
    bsf   INTCON,7			; Globale Interrupt Freigabe
    bsf   INTCON,6			; externe Interrupt Freigabe
    bsf   PIE1,0			; Timer 1 Interrupt enable
    bcf   TRISC,5			; Port C5 als Ausgabe deklariere
	bcf   TRISC,4			; SCL ist Ausgang
    bcf   TRISC,3			; SDA ist Ausganf
    bcf   TRISC,6			; ID ist Ausgang
	bcf   TRISC,7			; SC ist Ausgang
    bcf   TRISB,3			; US_S ist Ausgang
    bsf   TRISB,4           ; US_E ist Eingang
    movlw B'10000000'       ; V
    movwf OPTION_REG
    bcf   STATUS,RP0 		; PORTC ist in BankK !
    bsf   US_S				; Ultraschall sperren
    return
;****************************************************************************
; ini_lcd initialisiert das LCD 
;
;****************************************************************************

ini_lcd:
	bsf		SC			; chip Select an
    bcf		SCL			; clock low
	bcf		SDA			; daten low
	bcf		ID			; Instructions

	bcf		SC			;chip select
;	movlw	B'00111101'	; Function Set mit doppelte Haehe
  	movlw	B'00111001'	; Function Set Einfache Hhe
	call    spi_tx
	movlw	B'00011100'	; BIAS Set
	call    spi_tx
	movlw	B'01010010'	; Power Control
	call    spi_tx
	movlw	B'01101001'	; Spannungsfolger
	call    spi_tx
	movlw	B'011101010'; Contrast Set
	call    spi_tx
	movlw	B'00001100'	; Display on
	call    spi_tx
   	movlw	B'00000001'	; Curser RESET
	call    spi_tx
	bsf		ID			; Daten 
; Ausgabe ___ cm Fllst.

	movlw	B'00000000'	; Sende 'Blaenk'
	call    spi_tx
	movlw	B'00000000'	; Sende 'Blaenk'
	call    spi_tx
	movlw	B'00000000'	; Sende 'Blaenk'
	call    spi_tx
    movlw	B'01100011'	; Sende 'c'
	call    spi_tx
	movlw	B'01101101'	; Sende 'm'
	call    spi_tx
	movlw	B'00000000'	; Sende ' '
	call    spi_tx
	movlw	B'01000110'	; Sende 'F'
	call    spi_tx
	movlw	B'10000001'	; Sende ''
	call    spi_tx
	movlw	B'01101100'	; Sende 'l'
	call    spi_tx
	movlw	B'01101100'	; Sende 'l'
	call    spi_tx
	movlw	B'01110011'	; Sende 's'
	call    spi_tx
	movlw	B'01110100'	; Sende 't'
	call    spi_tx
	movlw	B'01100001'	; Sende 'a'
	call    spi_tx
	movlw	B'01101110'	; Sende 'n'
	call    spi_tx
	movlw	B'01100100'	; Sende 'n'
	call    spi_tx
	bsf		SC
    return
	

spi_tx:
	movwf	buf
	movlw	8
	movwf	count		; 8 Bits
	rrf		buf, 1		; shift right in preparation for next loop
tx_loop:
    nop
    bcf     SCL
	rlf		buf, 1		; shift bits out of buffer
	btfsc	buf, 7		; send current "bit 7"
	goto	tx_high
	bcf		SDA
	goto	tx_low
tx_high:		
	bsf		SDA
tx_low:	
    nop
    nop
	bsf		SCL			; pull clock high after sending bit
   	decfsz	count, 1	; alle 8 Bit bertragen
	goto	tx_loop
    movlw 	D'150'		; Wartezeit 150*10us 
    movwf 	us10_count
    call  	us10_wait	; 1,5 ms Warten
	return 

;****************************************************************************
; Wartefunktionen
; us10_wait wartet 10us*Wert in us10_count
; ms_wait wartet 1ms*Wert in ms_count
; s_Wait wastet 1s * wert in s_count
;****************************************************************************


us10_wait:
   movlw D'15'
   movwf temp2 
us10_loop:
   decfsz temp2,1
   goto us10_loop
   decfsz us10_count,1
   goto us10_wait
   return

ms_wait:
   movlw D'99'
   movwf us10_count
   call us10_wait
   decfsz ms_count,1
   goto ms_wait
   return

s_wait:
   movlw D'4'
   movwf temp3
s_wait_loop:
   movlw D'250'
   movwf ms_count
   call ms_wait
   decfsz temp3,1
   goto s_wait_loop
   decfsz s_count,1
   goto s_wait
   return


;****************************************************************************
; Messen: Durchfhrung einer Ultraschallmessung 
; 
;****************************************************************************

messung:
   clrf    err_flag	    ; Fehlerflag ruecksetzen
   clrf    level		; Laufzeitzaehler Rchsetzen
   bcf 	   US_S			; Freigabe fr Ultraschallpulse
   movlw   CMC		    ; Wert fr Timer
   movwf   TMR0		    ; Timer Zaehler setzen 
   bsf     INTCON,5		; Freigabe der Interrupts Timer 0
   movlw   D'25'        ; 
   movwf   us10_count   ; Breite des Ultraschallpulses (250 us)
   call    us10_wait    ; Warten
   bsf 	   US_S			; Stoppen des Ultraschallpulse
   movlw   D'100'        ;
   movwf   us10_count   ; Warten bis Nahfeldecho abgeklungen (1000 us)
   call    us10_wait    ; Warten
warten:
   movfw    level       
   sublw    D'250'     ; prfen ob Level > 250
   btfss    STATUS,0	; Prfen ob Ergebnis negativ
   goto     err_short
   btfss    US_E 
   goto 	warten      ; Warten bis Echo kommt
   goto     ready
err_short:
   movlw    B'01010011'  ;
   movwf    err_mod      
   bsf      err_flag,0    ; setzen des Fehlerflagd
ready:
   bcf      INTCON,5	; Sperren der Interrupts Timer 0  
   movfw    level       
   sublw    D'25'       ; Abstand muss > 20 cm sein
   btfsc    STATUS,0	; Prfen ob Ergebnis negativ
   goto     err_long   
   return 
err_long:
   bsf      err_flag,0    ; setzen des Fehlerflagd
   movlw    B'01001100'  ;
   movwf    err_mod   
   return

messen:
   call		messung
   btfsc    err_flag,0  ; Prfen ob Fehlerflag gesetzt wurde
   goto     fehler
   goto     weiter      ; Messung korrekt weiter mit Ausgabe
fehler:
   call     display_error   ; err Ausgabe
   bsf      LED			; LED aein 
   return
weiter:
   movlw	UE			; unter Enstellung laden
						; die Messung kann auch mehrfach durchgefhrt werden
                        ; um eine bessere Genauigkeit zu erhalten
   movwf    true_level  ; 
   movfw    level
   subwf	true_level	; Tatschlicher Pegel
   btfss    STATUS,0	; Prfen ob Fllstand Negativ
   clrf		true_level

   call display
   return
   END